package com.huimv.produce.produce.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.produce.produce.repo.*;
import com.huimv.produce.produce.service.IWeight;
import com.huimv.produce.produce.utils.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import java.math.BigDecimal;
import java.text.ParseException;
import java.util.*;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
public class WeightImpl implements IWeight {
    @Autowired
    private ProdDayWeightRepo dayWeightRepo;
    @Autowired
    private ProdBatchWeightRepo batchWeightRepo;
    @Autowired
    private ProdWeightRepo weightRepo;
    @Autowired
    private DateUtil dateUtil;

    /**
     * @Method : getDayWeight
     * @Description :
     * @Params : [farmId, startDate, endDate]
     * @Return : com.huimv.common.utils.Result
     * @Author : ZhuoNing
     * @Date : 2021/12/7
     * @Time : 13:58
     */
    @Override
    public Result getDayWeight(Integer farmId, String startDate, String endDate) {
        //
        List<ProdDayWeightEntity> dayWeightEntityList = dayWeightRepo.findByFarmIdAndStartAndEndDate(farmId, startDate, endDate);
        if (dayWeightEntityList.size() > 0) {
            BigDecimal totalBd = new BigDecimal(0);
            for (int a = 0; a < dayWeightEntityList.size(); a++) {
                ProdDayWeightEntity dayWeightEntity = dayWeightEntityList.get(a);
                totalBd = new BigDecimal(dayWeightEntity.getNetWeight()).add(totalBd);
            }
            JSONObject outJo = new JSONObject();
            outJo.put("total", totalBd.toString());
            outJo.put("weight", dayWeightEntityList);
            return new Result(ResultCode.SUCCESS, outJo);
        } else {
            return new Result(10001, "暂无数据.", false);
        }
    }

    /**
     * @Method : getDayWeightLastNtimes
     * @Description :
     * @Params : [farmId, days]
     * @Return : com.huimv.common.utils.Result
     * @Author : ZhuoNing
     * @Date : 2021/12/7
     * @Time : 13:58
     */
    @Override
    public Result getDayWeightLastNtimes(Integer farmId, Integer days) {
        //
        List<ProdDayWeightEntity> dayWeightEntityList = dayWeightRepo.findByFarmIdAndDays(farmId, days);
        if (dayWeightEntityList.size() > 0) {
            //排序
            List outList = new ArrayList();
            for (int a = dayWeightEntityList.size() - 1; a >= 0; a--) {
                outList.add(dayWeightEntityList.get(a));
            }
            //计算总重
            BigDecimal totalBd = new BigDecimal(0);
            for (int a = 0; a < dayWeightEntityList.size(); a++) {
                ProdDayWeightEntity dayWeightEntity = dayWeightEntityList.get(a);
                totalBd = new BigDecimal(dayWeightEntity.getNetWeight()).add(totalBd);
            }
            JSONObject outJo = new JSONObject();
            outJo.put("total", totalBd.toString());
            outJo.put("weight", outList);
            return new Result(ResultCode.SUCCESS, outJo);
        } else {
            return new Result(10001, "暂无数据.", false);
        }
    }

    /**
     * @Method : getWeightOnPage
     * @Description : 分页查询称重记录
     * @Params : [farmId, startDate, endDate, pageNo, pageSize]
     * @Return : com.huimv.common.utils.Result
     * @Author : ZhuoNing
     * @Date : 2021/12/7
     * @Time : 14:56
     */
    @Override
    public Result getWeightOnPage(Integer farmId, Date startDate, Date endDate, Integer pageNo, Integer pageSize) {
        Specification<ProdDayWeightEntity> sf = (Specification<ProdDayWeightEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            if (null != farmId) {
                predList.add(criteriaBuilder.equal(root.get("farmId").as(Integer.class), farmId));
            }
            if (null != startDate) {
                predList.add(criteriaBuilder.greaterThanOrEqualTo(root.get("addTime").as(Date.class), startDate));
            }
            if (null != endDate) {
                predList.add(criteriaBuilder.lessThanOrEqualTo(root.get("addTime").as(Date.class), endDate));
            }
            //
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
//        return new Result(ResultCode.SUCCESS,applyRepo.listApply(pageable,applyId));
        return new Result(ResultCode.SUCCESS, dayWeightRepo.findAll(sf, pageable));
    }

    /**
     * @Method : getBatchWeightOnPage
     * @Description : 分页查询批次重量记录
     * @Params : [farmId, addTime, pageNo, pageSize]
     * @Return : com.huimv.common.utils.Result
     * @Author : ZhuoNing
     * @Date : 2021/12/7
     * @Time : 20:31
     */
    @Override
    public Result getBatchWeightOnPage(Integer farmId, Date addTime, Integer pageNo, Integer pageSize) {
        Specification<ProdBatchWeightEntity> sf = (Specification<ProdBatchWeightEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            if (null != farmId) {
                predList.add(criteriaBuilder.equal(root.get("farmId").as(Integer.class), farmId));
            }
            if (null != addTime) {
                predList.add(criteriaBuilder.greaterThanOrEqualTo(root.get("addTime").as(Date.class), addTime));
            }
            //
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return new Result(ResultCode.SUCCESS, batchWeightRepo.findAll(sf, pageable));
    }

    /**
     * @Method : getBatchWeightOnPage
     * @Description :
     * @Params : [farmId, startDate, endDate, pageNo, pageSize]
     * @Return : com.huimv.common.utils.Result
     * @Author : ZhuoNing
     * @Date : 2021/12/7
     * @Time : 20:36
     */
    @Override
    public Result getBatchWeightOnPage(Integer farmId, Date startDate, Date endDate, Integer pageNo, Integer pageSize) throws ParseException {
        System.out.println("startDate 2>>" + startDate + " " + dateUtil.formatDateText(startDate));
        System.out.println("endDate 2>>" + endDate + " " + dateUtil.formatDateText(endDate));
        Specification<ProdBatchWeightEntity> sf = (Specification<ProdBatchWeightEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            if (null != farmId) {
                predList.add(criteriaBuilder.equal(root.get("farmId").as(Integer.class), farmId));
            }
            if (null != startDate) {
                predList.add(criteriaBuilder.greaterThanOrEqualTo(root.get("addTime").as(Date.class), startDate));
            }
            if (null != endDate) {
                predList.add(criteriaBuilder.lessThanOrEqualTo(root.get("addTime").as(Date.class), endDate));
            }
//            //
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return new Result(ResultCode.SUCCESS, batchWeightRepo.findAll(sf, pageable));
    }

    /**
     * @Method : getBatchWeightOnPage2
     * @Description :
     * @Params : [farmId, startDate, endDate, pageNo, pageSize]
     * @Return : com.huimv.common.utils.Result
     * @Author : ZhuoNing
     * @Date : 2021/12/8
     * @Time : 10:43
     */
    @Override
    public Result getBatchWeightOnPage2(Integer farmId, Date startDate, Date endDate, Integer pageNo, Integer pageSize) throws ParseException {
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize, Sort.Direction.DESC, "id");
        //
        return new Result(ResultCode.SUCCESS, batchWeightRepo.findByFarmIdAndStartDateAndEndDate(farmId, startDate, endDate, pageable));
    }

    /**
     * @Method      : getEveryTimeWeightOnPage
     * @Description : 
     * @Params      : [farmId, startDate, endDate, pageNo, pageSize]
     * @Return      : com.huimv.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/12/22       
     * @Time        : 19:41
     */
    @Override
    public Result getEveryTimeWeightOnPage(Integer farmId, Date startDate, Date endDate, Integer pageNo, Integer pageSize) throws ParseException {
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize, Sort.Direction.DESC, "id");
        //
        return new Result(ResultCode.SUCCESS, weightRepo.findWeightByFarmIdAndStartDateAndEndDate(farmId, startDate, endDate, pageable));
    }

    @PersistenceContext
    EntityManager entityManager;

    @Override
    public Result getBatchWeightOnPage3(Integer farmId, Date startDate, Date endDate, Integer pageNo, Integer pageSize) {
        StringBuilder countSelectSql = new StringBuilder();
        countSelectSql.append("select count(*) from ProdBatchWeightEntity bw where 1=1 ");

        StringBuilder selectSql = new StringBuilder();
        selectSql.append("from ProdBatchWeightEntity bw where 1=1 ");

        Map<String, Object> params = new HashMap<>();
        StringBuilder whereSql = new StringBuilder();
        //farmId
        if (farmId != null) {
            whereSql.append(" and farmId=:farmId ");
            params.put("farmId", farmId);
        }
        //startDate
        if(startDate != null){
            whereSql.append(" and addTime>=:startDate ");
            params.put("startDate", startDate);
        }
        //endDate
        if(endDate != null){
            whereSql.append(" and addTime<=:endDate ");
            params.put("endDate", endDate);
        }
        //加入排序顺序
        whereSql.append(" ORDER BY id DESC");
        String countSql = new StringBuilder().append(countSelectSql).append(whereSql).toString();
        Query countQuery = this.entityManager.createQuery(countSql, Long.class);
        this.setParameters(countQuery, params);
        Long count = (Long) countQuery.getSingleResult();

        String querySql = new StringBuilder().append(selectSql).append(whereSql).toString();
        Query query = this.entityManager.createQuery(querySql, ProdBatchWeightEntity.class);
        this.setParameters(query, params);
        query.setFirstResult((pageNo - 1) * pageSize);
        query.setMaxResults(pageSize);
        List<ProdBatchWeightEntity> batchWeightList = query.getResultList();

//        Pageable pageable = PageRequest.of(pageNo - 1, pageSize, Sort.Direction.DESC, "id");
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        Page<ProdBatchWeightEntity> incomeDailyPage = new PageImpl<ProdBatchWeightEntity>(batchWeightList, pageable, count);

        return new Result(ResultCode.SUCCESS, incomeDailyPage);
    }

    private void setParameters(Query query, Map<String, Object> params) {
        for (Map.Entry<String, Object> entry : params.entrySet()) {
            query.setParameter(entry.getKey(), entry.getValue());
        }
    }

    @Override
    public Result getBatchWeightOnPage4(Integer farmId, String startDate, String endDate, Integer pageNo, Integer pageSize) {
        StringBuilder countSelectSql = new StringBuilder();
        countSelectSql.append("select count(*) from prod_batch_weight bw where 1=1 ");

        StringBuilder selectSql = new StringBuilder();
        selectSql.append("from prod_batch_weight bw where 1=1 ");

        Map<String, Object> params = new HashMap<>();
        StringBuilder whereSql = new StringBuilder();
        //farmId
        if (farmId != null) {
            whereSql.append(" AND farm_id= :farmId");
            params.put("farmId", farmId);
        }
        //startDate
        if(startDate != null){
            whereSql.append(" AND date_format(add_time,'%Y-%m-%d')>= :startDate ");
            params.put("startDate", startDate);
        }
        //endDate
        if(endDate != null){
            whereSql.append(" AND date_format(add_time,'%Y-%m-%d')<= :endDate");
            params.put("endDate", endDate);
        }
        //加入排序顺序
        whereSql.append(" ORDER BY id DESC");
        String countSql = new StringBuilder().append(countSelectSql).append(whereSql).toString();
        Query countQuery = this.entityManager.createNativeQuery(countSql);
        this.setParameters(countQuery, params);
//        Long count = (Long) countQuery.getSingleResult();
        int size = countQuery.getResultList().size();
//        System.out.println("count>>"+count);
        System.out.println("size>>"+size);

        String querySql = new StringBuilder().append(selectSql).append(whereSql).toString();
        Query query = this.entityManager.createNativeQuery(querySql);
        this.setParameters(query, params);
        query.setFirstResult((pageNo - 1) * pageSize);
        query.setMaxResults(pageSize);
        List<ProdBatchWeightEntity> batchWeightList = query.getResultList();
//        Pageable pageable = PageRequest.of(pageNo - 1, pageSize, Sort.Direction.DESC, "id");
//        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
//        Page<ProdBatchWeightEntity> incomeDailyPage = new PageImpl<ProdBatchWeightEntity>(batchWeightList, pageable);

        return new Result(ResultCode.SUCCESS, batchWeightList);
    }
}
